package com.ym.mec.teacher.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.dayaedu.cbs.openfeign.client.CoursewareFeignService;
import com.dayaedu.cbs.openfeign.wrapper.courseware.CbsLessonCoursewareDetailWrapper;
import com.dayaedu.cbs.openfeign.wrapper.courseware.CbsLessonCoursewareWrapper;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.entity.Subject;
import com.ym.mec.biz.dal.entity.SysMusicScore;
import com.ym.mec.biz.dal.entity.Teacher;
import com.ym.mec.biz.dal.enums.ClientEnum;
import com.ym.mec.biz.dal.wrapper.LessonCoursewareWrapper;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RequestMapping("${app-config.url.teacher:}/lessonCourseware")
@Api(tags = "课件教材")
@RestController
public class LessonCoursewareController extends BaseController {

	@Resource
	private CoursewareFeignService coursewareFeignService;
	@Resource
	private SysUserService sysUserService;
	@Resource
	private LessonCoursewareService lessonCoursewareService;
	@Resource
	private OrganizationService organizationService;
	@Resource
	private SysMusicScoreService sysMusicScoreService;
	@Resource
	private CbsMusicScoreCategoriesService cbsMusicScoreCategoriesService;
	@Resource
	private TeacherDao teacherDao;

	@ApiOperation(value = "分页查询已添加的课件")
	@PostMapping("/queryLessonCourseware")
	public HttpResponseResult<PageInfo<LessonCoursewareWrapper.LessonCoursewareDto>> queryLessonCourseware(@RequestBody LessonCoursewareWrapper.LessonCoursewareQuery query){
		//判断用户是否可以访问云练习
		Integer userId = sysUserService.getUserId();
		Teacher teacher = teacherDao.get(userId);
		if (teacher == null) {
			throw new BizException("老师信息不存在");
		}
		//用户是否可以使用云课堂
		if(teacher.getCoursewareFlag()){
			throw new BizException("您没有权限使用云课堂");
		}
		query.setEnable(true);
		query.setTeacherId(userId);
		IPage<LessonCoursewareWrapper.LessonCoursewareDto> pages = lessonCoursewareService.selectPage(QueryInfo.getPage(query), query);
		return succeed(PageUtil.pageInfo(pages));
	}

	@ApiOperation(value = "获取课件详情")
	@GetMapping("/getLessonCoursewareDetail/{id}")
	public HttpResponseResult<CbsLessonCoursewareWrapper.LessonCourseware> getLessonCoursewareDetail(@PathVariable("id") Long id){
		return succeed(coursewareFeignService.lessonCoursewareDetail(id).feignData());
	}

	@ApiOperation(value = "获取课件关联的课程列表")
	@GetMapping("/getLessonCoursewareCourseList/{id}")
	public HttpResponseResult<List<LessonCoursewareWrapper.CourseScheduleCoursewareDetail>> getLessonCoursewareCourseList(@PathVariable("id") Long id,
																														  String search,
																														  Boolean detailFlag){
		CbsLessonCoursewareWrapper.CoursewareDetailQuery query = new CbsLessonCoursewareWrapper.CoursewareDetailQuery();
		query.setId(id);
		query.setSearch(search);
		query.setDetailFlag(detailFlag);
		List<CbsLessonCoursewareWrapper.CourseScheduleCoursewareDetail> courseScheduleCoursewareDetails = coursewareFeignService.coursewareDetail(query).feignData();
		if (CollectionUtils.isNotEmpty(courseScheduleCoursewareDetails)) {
			SysUser sysUser = sysUserService.getUser();
			String teachingMaterialIds = organizationService.getTeachingMaterialIds(sysUser.getOrganId().toString(),sysUser.getTenantId());
			List<Integer> categoryIdList = cbsMusicScoreCategoriesService.getAllCategoryIdList(teachingMaterialIds);
			LessonCoursewareWrapper.LessonCoursewareDetailQuery query1 = new LessonCoursewareWrapper.LessonCoursewareDetailQuery();
			query1.setUserType(ClientEnum.TEACHER);
			query1.setTenantCategoriesIdList(categoryIdList);
			//获取课件下面包含的所有的曲目编号
			List<Long> musicScoreIds = lessonCoursewareService.getMusicScoreIds(courseScheduleCoursewareDetails);
			Map<Long, SysMusicScore> musicScoreMap = new HashMap<>();
			if (CollectionUtils.isNotEmpty(musicScoreIds)) {
				List<SysMusicScore> musicScores = sysMusicScoreService.getDao().findByCbsId(musicScoreIds,false);
				musicScoreMap = musicScores.stream().collect(Collectors.toMap(e -> Long.parseLong(e.getCbsMusicSheetId()), o -> o));
			}
			for (CbsLessonCoursewareWrapper.CourseScheduleCoursewareDetail e : courseScheduleCoursewareDetails) {
				lessonCoursewareService.setTreeResourceId(e.getKnowledgePointList(), query1, musicScoreMap);
			}
			courseScheduleCoursewareDetails.removeIf(courseScheduleCoursewareDetail -> !filterAndUpdateKnowledgePointList(courseScheduleCoursewareDetail));
			String jsonString = JSONObject.toJSONString(courseScheduleCoursewareDetails);
			List<LessonCoursewareWrapper.CourseScheduleCoursewareDetail> courseScheduleCoursewareDetailList =
					JSONObject.parseArray(jsonString, LessonCoursewareWrapper.CourseScheduleCoursewareDetail.class);
			return succeed(courseScheduleCoursewareDetailList);
		}
		return succeed(new ArrayList<>());
	}

	private boolean filterAndUpdateKnowledgePointList(CbsLessonCoursewareWrapper.CourseScheduleCoursewareDetail courseScheduleCoursewareDetail) {
		List<CbsLessonCoursewareDetailWrapper.KnowledgePointSmall> knowledgePointList = courseScheduleCoursewareDetail.getKnowledgePointList();
		if (CollectionUtils.isEmpty(knowledgePointList)) {
			return false; // 如果 knowledgePointList 为空，直接返回 false，后续会删除该 courseScheduleCoursewareDetail
		}

		// 过滤 KnowledgePointSmall
		knowledgePointList.removeIf(knowledgePointSmall -> !filterKnowledgePoint(knowledgePointSmall));
		courseScheduleCoursewareDetail.setKnowledgePointList(knowledgePointList);

		// 如果过滤后的 knowledgePointList 为空，返回 false
		return CollectionUtils.isNotEmpty(knowledgePointList);
	}

	private boolean filterKnowledgePoint(CbsLessonCoursewareDetailWrapper.KnowledgePointSmall knowledgePointSmall) {
		// 过滤 materialList
//		if (CollectionUtils.isNotEmpty(knowledgePointSmall.getMaterialList())) {
//			filterAndUpdateMaterialList(knowledgePointSmall.getMaterialList(), search);
//		}
		// 过滤子知识点 children
		if (CollectionUtils.isNotEmpty(knowledgePointSmall.getChildren())) {
			knowledgePointSmall.getChildren().removeIf(child -> !filterKnowledgePoint(child));
		}
		// 保留当前的条件：materialList 不为空 或者 children 不为空，**否则删除当前节点**
		return CollectionUtils.isNotEmpty(knowledgePointSmall.getMaterialList()) ||
				CollectionUtils.isNotEmpty(knowledgePointSmall.getChildren());
	}

	private void filterAndUpdateMaterialList(List<CbsLessonCoursewareDetailWrapper.MaterialSmall> materialList, String search) {
		materialList.removeIf(materialSmall -> !materialSmall.getName().contains(search));
	}

	@ApiOperation(value = "获取课程详情")
	@GetMapping("/getLessonCourseDetail/{id}")
	public HttpResponseResult<CbsLessonCoursewareDetailWrapper.LessonCoursewareDetail> getLessonCourseDetail(@PathVariable("id") Long id){
		SysUser sysUser = sysUserService.getUser();
		String teachingMaterialIds = organizationService.getTeachingMaterialIds(sysUser.getOrganId().toString(),sysUser.getTenantId());
		List<Integer> categoryIdList = cbsMusicScoreCategoriesService.getAllCategoryIdList(teachingMaterialIds);
        LessonCoursewareWrapper.LessonCoursewareDetailQuery query = new LessonCoursewareWrapper.LessonCoursewareDetailQuery();
        query.setLessonCoursewareDetailId(id);
        query.setUserType(ClientEnum.TEACHER);
        query.setTenantCategoriesIdList(categoryIdList);
		return succeed(lessonCoursewareService.getLessonCourseDetail(query));
	}

	@ApiOperation(value = "获取课件关联的声部列表")
	@GetMapping("/getLessonCoursewareSubjectList")
	public HttpResponseResult<List<Subject>> getLessonCoursewareSubjectList(){
		return succeed(lessonCoursewareService.getDao().getLessonCoursewareSubjectList());
	}

	@ApiOperation(value = "关联课程类型")
	@PostMapping("/refLevel")
	public HttpResponseResult<List<LessonCoursewareWrapper.RefLevelDetailInfo>> refLevel(@RequestBody LessonCoursewareWrapper.RefLevelQuery query) {
		query.setTeacherId(sysUserService.getUserId());
        query.setClient(ClientEnum.TEACHER);
		return succeed(lessonCoursewareService.refLevel(query));
	}
}
