package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ym.mec.biz.dal.entity.MemberRankSetting;
import com.ym.mec.biz.dal.entity.TenantMemberRankSetting;
import com.ym.mec.biz.dal.mapper.TenantMemberRankSettingMapper;
import com.ym.mec.biz.dal.wrapper.TenantMemberRankSettingWrapper;
import com.ym.mec.biz.service.MemberRankSettingService;
import com.ym.mec.biz.service.TenantMemberRankSettingService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 机构学练宝配置表
 * 2024-10-23 10:48:21
 */
@Slf4j
@Service
public class TenantMemberRankSettingServiceImpl extends ServiceImpl<TenantMemberRankSettingMapper, TenantMemberRankSetting> implements TenantMemberRankSettingService {

    @Autowired
    private MemberRankSettingService memberRankSettingService;

    @Override
    @Transactional
    public void update(Integer tenantId, List<TenantMemberRankSettingWrapper.TenantMemberRankSettingAdd> memberInfo) {

        // 先删除
        lambdaUpdate().eq(TenantMemberRankSetting::getTenantId, tenantId).remove();

        // 在新增
        List<TenantMemberRankSetting> tenantMemberRankSettings = JSON.parseArray(JSON.toJSONString(memberInfo), TenantMemberRankSetting.class);
        tenantMemberRankSettings.forEach(tenantMemberRankSetting -> tenantMemberRankSetting.setTenantId(tenantId));

        saveBatch(tenantMemberRankSettings);

    }

    @Override
    public List<TenantMemberRankSettingWrapper.TenantMemberRankSettingAdd> query(Integer tenantId) {
        List<TenantMemberRankSetting> list = lambdaQuery().eq(TenantMemberRankSetting::getTenantId, tenantId).list();
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>();
        }
        List<TenantMemberRankSettingWrapper.TenantMemberRankSettingAdd> tenantMemberRankSettingAdds = new ArrayList<>();
        for (TenantMemberRankSetting tenantMemberRankSetting : list) {
            TenantMemberRankSettingWrapper.TenantMemberRankSettingAdd add = new TenantMemberRankSettingWrapper.TenantMemberRankSettingAdd();
            add.setMemberRankSettingId(tenantMemberRankSetting.getMemberRankSettingId());
            add.setType(tenantMemberRankSetting.getType());
            add.setMemberConfig(JSON.parseObject(tenantMemberRankSetting.getMemberConfig(), TenantMemberRankSettingWrapper.MemberConfig.class));
            tenantMemberRankSettingAdds.add(add);
        }
//        List<TenantMemberRankSettingWrapper.TenantMemberRankSettingAdd> tenantMemberRankSettingAdds = JSON.parseArray(JSON.toJSONString(list), TenantMemberRankSettingWrapper.TenantMemberRankSettingAdd.class);
        // 学练宝ID集合
        List<Integer> ids = tenantMemberRankSettingAdds.stream().map(TenantMemberRankSettingWrapper.TenantMemberRankSettingAdd::getMemberRankSettingId).distinct().collect(Collectors.toList());
        List<MemberRankSetting> memberRankSettings = memberRankSettingService.listByIds(ids);
        // map
        Map<Integer, String> map =
            memberRankSettings.stream().collect(Collectors.toMap(MemberRankSetting::getId, MemberRankSetting::getName));

        tenantMemberRankSettingAdds
            .forEach(tenantMemberRankSettingAdd -> tenantMemberRankSettingAdd.setMemberRankSettingName(map.get(tenantMemberRankSettingAdd.getMemberRankSettingId())));

        return tenantMemberRankSettingAdds;
    }
}